﻿using System;
using System.IO;

namespace CutsceneCompiler
{
    public class Executable
    {
        private static readonly uint FileMagic = ('E' << 24) | ('X' << 16) | ('E' << 8) | 'C';

        public string Name { get; private set; }

        private string[] mStringTable;
        public string GetString(ushort idx) { return idx < mStringTable.Length ? mStringTable[idx] : null; }
        public ushort StringTableSize { get { return (ushort)mStringTable.Length; } }

        private byte[] mData;
        public byte this[ushort idx] { get { return mData[idx]; } }
        public ushort DataSize { get { return (ushort)mData.Length; } }

        public Executable(string name, string[] strings, byte[] data)
        {
            Name = name;

            mStringTable = strings;
            mData = data;
        }

        public bool Save(string filename)
        {
            bool success = false;

            try
            {
                FileStream stream = File.Create(filename);
                BinaryWriter writer = new BinaryWriter(stream);

                writer.Write(FileMagic);

                writer.Write((uint)mData.Length);
                writer.Write(mData, 0, mData.Length);

                writer.Write((ushort)mStringTable.Length);
                foreach (string str in mStringTable)
                    writer.Write(str);

                stream.Close();

                success = true;
            }
            catch
            {
            }

            return success;
        }

        public static Executable LoadFromFile(string filename)
        {
            Executable result = null;
            try
            {
                FileStream stream = File.OpenRead(filename);
                BinaryReader reader = new BinaryReader(stream);

                uint magic = reader.ReadUInt32();
                if (magic == FileMagic)
                {
                    int dataSize = (int)reader.ReadUInt32();
                    byte[] data = new byte[dataSize];
                    reader.Read(data, 0, dataSize);

                    int stringTableSize = (int)reader.ReadUInt16();
                    string[] stringTable = new string[stringTableSize];
                    for (int i = 0; i < stringTableSize; i++)
                        stringTable[i] = reader.ReadString();

                    string cutsceneName = Path.GetFileNameWithoutExtension(filename);

                    result = new Executable(cutsceneName, stringTable, data);
                }

                stream.Close();

            }
            catch
            {
            }

            return result;
        }
    }
}
