﻿using System;
using System.IO;
using System.Collections.Generic;

namespace CutsceneCompiler
{
    public class ExecutableWriter
    {
        public string Name { get; set; }

        private List<string> mStrings;
        private MemoryStream mData;

        public ushort DataLength { get { return (ushort)mData.Length; } }

        public ExecutableWriter(string name)
        {
            Name = name;
            mStrings = new List<string>();
            mData = new MemoryStream();
        }

        public int AddString(string str)
        {
            int idx = mStrings.IndexOf(str);
            if (idx < 0)
            {
                idx = mStrings.Count;
                mStrings.Add(str);
            }
            return idx;
        }

        public void AddOpCode(OpCode op)
        {
            mData.WriteByte((byte)op);
        }

        public void AddAddress(ushort addr)
        {
            mData.WriteByte((byte)(((int)addr & 0xff00) >> 8));
            mData.WriteByte((byte)((int)addr & 0x00ff));
        }

        public void InsertAddress(ushort offset, ushort addr)
        {
            long curPosition = mData.Position;
            mData.Seek(offset, SeekOrigin.Begin);

            mData.WriteByte((byte)(((int)addr & 0xff00) >> 8));
            mData.WriteByte((byte)((int)addr & 0x00ff));

            mData.Seek(curPosition, SeekOrigin.Begin);
        }

        public void AddLiteral(ushort val)
        {
            if (val < 0x007f)
            {
                mData.WriteByte((byte)val);
            }
            else
            {
                byte high = (byte)(((int)val & 0xff00) >> 8);
                byte low = (byte)((int)val & 0x00ff);

                if (val < 0x3fff)
                {
                    high |= 0x80;
                    mData.WriteByte(high);
                    mData.WriteByte(low);
                }
                else
                {
                    mData.WriteByte(0xc0);
                    mData.WriteByte(high);
                    mData.WriteByte(low);
                }
            }
        }

        public void AddDuration(byte seconds, ushort milliseconds)
        {
            AddLiteral((ushort)((int)seconds * 1000 + milliseconds));
        }

        public void AddDuration(ushort milliseconds)
        {
            AddLiteral(milliseconds);
        }

        public void AddExecutable(ExecutableWriter exe)
        {
            string[] strings = exe.mStrings.ToArray();
            mStrings.AddRange(strings);

            byte[] data = exe.mData.ToArray();
            mData.Write(data, 0, data.Length);
        }

        public Executable CreateExecutable()
        {
            return new Executable(Name, mStrings.ToArray(), mData.ToArray());
        }
    }
}
