﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;

namespace CutsceneEditor
{
    public class AnimatedGIF
    {
        private static readonly byte[] kGifAnimation = { 33, 255, 11, 78, 69, 84, 83, 67, 65, 80, 69, 50, 46, 48, 3, 1, 0, 0, 0 };

        // This code is based on: http://social.msdn.microsoft.com/Forums/vstudio/en-US/0c4252c8-8274-449c-ad9b-e4f07a8f8cdd/how-could-i-create-an-animated-gif-file-from-several-other-jpg-files-in-c-express?forum=csharpgeneral
        public static bool SaveFromFrames(string filename, List<Bitmap> aFrames, int framesPerSecond)
        {
            MemoryStream memStream = new MemoryStream();
            BinaryWriter gifWriter = new BinaryWriter(new FileStream(filename, FileMode.Create));

            // Convert the first frame into a GIF
            aFrames[0].Save(memStream, ImageFormat.Gif);
            byte[] aGifData = memStream.ToArray();

            // Setup the image header for an animated GIF
            aGifData[10] = (byte)(aGifData[10] & 0X78); // No global color table.
            gifWriter.Write(aGifData, 0, 13);
            gifWriter.Write(kGifAnimation);

            // Write the header for the first frame
            WriteFrameHeader(framesPerSecond, gifWriter, aGifData);

            // Write the remaining frames
            for (int i = 1; i < aFrames.Count; i++)
            {
                // Convert the current frame into a GIF
                memStream.SetLength(0);
                aFrames[i].Save(memStream, ImageFormat.Gif);
                aGifData = memStream.ToArray();

                WriteFrameHeader(framesPerSecond, gifWriter, aGifData);
            }

            // Write the image footer...
            gifWriter.Write(aGifData[aGifData.Length - 1]);
            // ...and clean up
            gifWriter.Close();
            memStream.Dispose();

            return true;
        }

        private static void WriteFrameHeader(int framesPerSecond, BinaryWriter gifWriter, byte[] aGifData)
        {
            // Patch the frame header...
            aGifData[785] = (byte)framesPerSecond;
            aGifData[786] = 0;
            aGifData[798] = (byte)(aGifData[798] | 0X87);
            // ...and write it to the file
            gifWriter.Write(aGifData, 781, 18);
            gifWriter.Write(aGifData, 13, 768);
            gifWriter.Write(aGifData, 799, aGifData.Length - 800);
        }
    }
}
