﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Reflection;
using System.IO;

namespace CutsceneEditor
{
    public class ImageSequence
    {
        private List<Image> mImages;

        public int NumImages { get { return mImages.Count; } }

        public Image this[int idx] { get { return mImages[idx % mImages.Count]; } }

        public ImageSequence()
        {
            mImages = new List<Image>();
        }

        public void AddImageFromResource(string name)
        {
            Assembly assembly = Assembly.GetEntryAssembly();
            List<string> resourceNames = new List<string>(assembly.GetManifestResourceNames());

            string baseName = assembly.ManifestModule.Name.Replace(".exe", ".Content.");
            string imageName = baseName + name + ".png";

            if (resourceNames.Contains(imageName))
            {
                Bitmap img = new Bitmap(assembly.GetManifestResourceStream(imageName));
                mImages.Add(img);
            }
        }

        public bool AddImageFromFile(string filename)
        {
            bool success = false;

            if (File.Exists(filename))
            {
                Bitmap img = Bitmap.FromFile(filename) as Bitmap;
                if (img != null)
                {
                    mImages.Add(img);
                    success = true;
                }
            }

            return success;
        }
    }
}
