﻿using System;
using CutsceneCompiler;

namespace CutsceneEditor
{
    public class Routine
    {
        public enum Registers
        {
            A,
            B,
            C,
            D,

            PC,
            Status,
        }

        [Flags]
        public enum StatusFlags : ushort
        {
            Error = 0x0001,
            Yield = 0x0002,
        }

        public class ExecutionContext
        {
            public Script Script { get; private set; }
            public Scene Scene { get; private set; }
            public Executable Executable { get; private set; }

            private ushort[] mRegisters;
            public ushort this[Registers reg]
            {
                get { return mRegisters[(int)reg]; }
                set { mRegisters[(int)reg] = value; }
            }

            internal ExecutionContext(Script script, ushort pc)
            {
                Script = script;
                Scene = script.Scene;
                Executable = script.Executable;

                int numRegisters = Enum.GetNames(typeof(Registers)).Length;
                mRegisters = new ushort[numRegisters];
                this[Registers.PC] = pc;
            }

            public void SetStatus(StatusFlags flag)
            {
                this[Registers.Status] |= (ushort)flag;
            }

            public void UnsetStatus(StatusFlags flag)
            {
                this[Registers.Status] = (ushort)((int)this[Registers.Status] & ~(int)flag);
            }

            public bool GetStatus(StatusFlags flag)
            {
                return (this[Registers.Status] & (ushort)flag) != 0;
            }

            public void ClearStatus()
            {
                this[Registers.Status] = 0;
            }

            public override string ToString()
            {
                string state = "";

                state += "PC=" + this[Registers.PC].ToString();
                state += " Status=" + this[Registers.Status].ToString();
                state += " A=" + this[Registers.A].ToString();
                state += " B=" + this[Registers.B].ToString();
                state += " C=" + this[Registers.C].ToString();
                state += " D=" + this[Registers.D].ToString();

                return state;
            }
        }

        public string Name { get; private set; }

        public ExecutionContext Context { get; private set; }
        public bool IsDone { get; private set; }

        public Routine(Script script, ushort pc)
        {
            Name = script.Executable.GetString(script.Executable[pc++]);

            Context = new Routine.ExecutionContext(script, pc);

            IsDone = false;
        }

        public void Execute()
        {
            while (!IsDone)
            {
                Context.ClearStatus();

                if (Interpreter.ExecuteOp(Context))
                {
                    if (Context.GetStatus(StatusFlags.Yield))
                        break;
                }
                else
                    IsDone = true;

                if (Context.GetStatus(StatusFlags.Error))
                    IsDone = true;
            }
        }

        public override string ToString()
        {
            return Name;
        }
    }
}
