﻿using System;
using System.Collections.Generic;

namespace CutsceneEditor
{
    public class Scene
    {
        private Dictionary<string, Actor> mActors;

        public IEnumerable<Actor> Actors
        {
            get
            {
                foreach (string actorName in mActors.Keys)
                    yield return mActors[actorName];
            }
        }

        private Dictionary<ushort, Text> mTexts;

        public IEnumerable<Text> Texts
        {
            get
            {
                foreach (ushort textID in mTexts.Keys)
                    yield return mTexts[textID];
            }
        }

        public Scene()
        {
            mActors = new Dictionary<string, Actor>();
            mTexts = new Dictionary<ushort, Text>();
        }

        public void Reset()
        {
            mActors.Clear();
            mTexts.Clear();
            Text.TextCount = 0;
        }

        public void AddActor(Actor actor)
        {
            mActors.Add(actor.Name, actor);
        }

        public void RemoveActor(Actor actor)
        {
            mActors.Remove(actor.Name);
        }

        public Actor GetActor(string name)
        {
            Actor actor;
            if (mActors.TryGetValue(name, out actor))
                return actor;

            return null;
        }

        public void AddText(Text text)
        {
            mTexts.Add(text.ID, text);
        }

        public void RemoveText(Text text)
        {
            mTexts.Remove(text.ID);
        }

        public Text GetText(ushort textID)
        {
            Text text;
            if (mTexts.TryGetValue(textID, out text))
                return text;

            return null;
        }
    }
}
